/************************************************************************/
/*                                                                      */
/*                      RC5 Remote Receiver                             */
/*                                                                      */
/*              Author: Peter Dannegger                                 */
/*                      danni@specs.de                                  */
/*                                                                      */
/************************************************************************/
#include "main.h"
//#include <util/delay.h>
//#include <io.h>
//#include <interrupt.h>
//#include <signal.h>
//#include <stdlib.h>

#define RC5TIME 	1.778e-3		// 1.778msec
#define PULSE_MIN	(uchar)(XTAL / 512 * RC5TIME * 0.4 + 0.5)
#define PULSE_1_2	(uchar)(XTAL / 512 * RC5TIME * 0.8 + 0.5)
#define PULSE_MAX	(uchar)(XTAL / 512 * RC5TIME * 1.2 + 0.5)


//Macro Definitionen zum Einlesen der Zustnde an den Pins wo der Drehencoder angeschlossen ist.
#define PHASE_A	(PINB & 1<<PINB0)	// PINB.0
#define PHASE_B (PINB & 1<<PINB1)	// PINB.1


uchar	rc5_bit;				// bit value
uchar	rc5_time;				// count bit time
uint	rc5_tmp;				// shift bits in
volatile uint	rc5_data;				// store result

volatile char	enc_delta;		// -128 ... 127		//fr Drehencoder


ISR (TIMER0_OVF0_vect)	//Timer/Counter0 Overflow 
{

  static char enc_last = 0x01;		//fr Drehencoder
  char i = 0;						//fr Drehencoder


  uint tmp = rc5_tmp;				// for faster access

  TCNT0 = -2;					// 2 * 256 = 512 cycle
  
//  		PORTB ^= (1 << PB1);    // LED toggeln PB1
//				 _delay_ms( 25 );

  if( ++rc5_time > PULSE_MAX ){			// count pulse time
    if( !(tmp & 0x4000) && tmp & 0x2000 )	// only if 14 bits received
      rc5_data = tmp;
    tmp = 0;
  }

  if( (rc5_bit ^ xRC5_IN) & 1<<xRC5 ){		// change detect
    rc5_bit = ~rc5_bit;				// 0x00 -> 0xFF -> 0x00

    if( rc5_time < PULSE_MIN )			// to short
      tmp = 0;

    if( !tmp || rc5_time > PULSE_1_2 ){		// start or long pulse time
      if( !(tmp & 0x4000) )			// not to many bits
        tmp <<= 1;				// shift
      if( !(rc5_bit & 1<<xRC5) )		// inverted bit
        tmp |= 1;				// insert new bit
      rc5_time = 0;				// count next pulse time
    }
  }

  rc5_tmp = tmp;
  
//ab hier Drehencoder auswerten  
    if( PHASE_A )
    i = 1;

	if( PHASE_B )
    i ^= 3;				// convert gray to binary

	i -= enc_last;			// difference new - last

	if( i & 1 ){				// bit 0 = value (1)
    enc_last += i;			// store new as next last

    enc_delta += (i & 2) - 1;		// bit 1 = direction (+/-)
  }
//Ende Drehencoder auswerten  
  
}



